@@ -207,9 +207,14 @@ def img_show(item):
 def article_show(item):
     if item is None:
         return ""
-    return item.title
+    return constraint(item.title, 25)
 
 
+def constraint(value, limit):
+    if value is None or len(value) <= limit:
+        return value
+    return value[:limit - 1] + "…"
+
 def get_cache_info(table):
     try:
         with open(f"images/{table}/cache_info.json", 'r') as f:
