@@ -17,7 +17,7 @@ import sqlite3
 
 from tibiawikisql import schema
 from tibiawikisql.models import abc
-from tibiawikisql.utils import clean_links, convert_tibiawiki_position
+from tibiawikisql.utils import clean_links, convert_tibiawiki_position, find_template, strip_code
 
 price_to_template = re.compile(r"{{(?:NPC List\s*|Price to (?:Buy|Sell))\s*([^}]+)}}")
 npc_offers = re.compile(r"\|([^|:\[]+)(?::\s?(\d+))?(?:\s?\[\[([^\]]+))?")
@@ -28,38 +28,41 @@ spells_pattern = re.compile(r"\|([^|]+)")
 trades_sell_template = re.compile(r"{{Trades/Sells\s*(?:\|note=([^|]+))?([^}]+)}}")
 npc_trades = re.compile(r"\|([^|,\[]+)(?:,\s?([+-]?\d+))?(?:\s?\[\[([^\]]+))?")
 
-transport_template = re.compile(r"{{Transport\s*(?:\|discount=([^|]+))?([^}]+)}}")
-npc_destinations = re.compile(r"\|([^,]+),\s?(\d+)(?:;\s?([^|]+))?")
-
 ilink_pattern = re.compile(r"{{Ilink\|([^}]+)}}")
 
 
 def parse_destinations(value):
+    """Parse an NPC destinations into a list of tuples.
+
+    The tuple contains the  destination's name, price and notes.
+    Price and notes may not be present.
+
+    Parameters
+    ----------
+    value: :class:`str`
+        A string containing destinations.
+
+    Returns
+    -------
+    list(:class:`tuple`)
+        A list of tuples containing the parsed destinations.
     """
-        Parses an NPC destinations into a list of tuples.
-
-        The tuple contains the  destination's name, price and notes.
-        Price and notes may not be present.
-
-        Parameters
-        ----------
-        value: :class:`str`
-            A string containing destinations.
-
-        Returns
-        -------
-        list(:class:`tuple`)
-            A list of tuples containing the parsed destinations.
-        """
+    template = find_template(value, "Transport", partial=True)
+    if not template:
+        return []
     result = []
-    for __, destinations in transport_template.findall(value):
-        result.extend(npc_destinations.findall(destinations))
+    for param in template.params:
+        if param.showkey:
+            continue
+        data, *notes = strip_code(param).split(";", 1)
+        notes = notes[0] if notes else ''
+        destination, price = data.split(",")
+        result.append((destination, price, notes))
     return result
 
 
 def parse_item_offers(value):
-    """
-    Parses NPC item offers into a list of tuples.
+    """Parse NPC item offers into a list of tuples.
 
     The tuple contains the item's name, price and currency.
     Price and currency may not be present.
@@ -82,8 +85,7 @@ def parse_item_offers(value):
 
 
 def parse_item_trades(value):
-    """
-    Parses an NPC item trades into a list of tuples.
+    """Parse an NPC item trades into a list of tuples.
 
     The tuple contains the item's name, price and currency.
     Price and currency may not be present.
@@ -106,7 +108,7 @@ def parse_item_trades(value):
 
 
 def parse_spells(value):
-    """Parses an NPC's teacheable spells.
+    """Parse an NPC's teacheable spells.
 
     Parameters
     ----------
@@ -126,7 +128,7 @@ def parse_spells(value):
 
 
 def replace_ilinks(value):
-    """Replaces the ILink template with a regular link.
+    """Replace the ILink template with a regular link.
 
     Parameters
     ----------
@@ -142,8 +144,7 @@ def replace_ilinks(value):
 
 
 class Npc(abc.Row, abc.Parseable, table=schema.Npc):
-    """
-    Represents a non-playable character.
+    """Represents a non-playable character.
 
     Attributes
     ----------
@@ -157,12 +158,10 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
         The in-game name of the NPC.
     gender: :class:`str`
         The gender of the NPC.
-    race: :class:`str`
-        The race of the NPC.
-    job: :class:`str`
-        The NPC's job.
-    job_additionals: :class:`str`
-        Other additional jobs the NPC might have, separated by commas.
+    races: list of :class:`str`
+        The races of the NPC.
+    jobs: list of :class:`str`
+        The jobs of the NPC.
     location: :class:`str`
         The location of the NPC.
     city: :class:`str`
@@ -183,20 +182,20 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
         Items sold by the NPC.
     buy_offers: list of :class:`NpcBuyOffer`
         Items bought by the NPC.
-    destinations: list of :class:`NpcSellOffer`
+    destinations: list of :class:`NpcDestination`
         Places where the NPC can travel to.
     teaches: list of :class:`NpcSpell`
         Spells this NPC can teach.
     """
+
     __slots__ = (
         "article_id",
         "title",
         "timestamp",
         "name",
         "gender",
-        "race",
-        "job",
-        "job_additionals",
+        "races",
+        "jobs",
         "location",
         "city",
         "x",
@@ -219,8 +218,6 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
         "actualname": ("name", str.strip),
         "location": ("location", clean_links),
         "gender": ("gender", str.strip),
-        "race": ("race", str.strip),
-        "job": ("job", str.strip),
         "city": ("city", str.strip),
         "posx": ("x", convert_tibiawiki_position),
         "posy": ("y", convert_tibiawiki_position),
@@ -228,26 +225,25 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
         "implemented": ("version", str.strip),
         "status": ("status", str.lower),
     }
-    _pattern = re.compile(r"Infobox[\s_]NPC")
+    _template = "Infobox_NPC"
 
     @property
-    def job_list(self):
-        if not self.job_additionals:
-            return [self.job] if self.job else []
-        return [self.job] + self.job_additionals.split(",")
+    def job(self):
+        """:class:`str`: Get the first listed job of the NPC, if any."""
+        return self.jobs[0] if self.jobs else None
+
+    @property
+    def race(self):
+        """:class:`str`: Get the first listed race of the NPC, if any."""
+        return self.races[0] if self.races else None
 
     @classmethod
     def from_article(cls, article):
-        npc = super().from_article(article)
-        additional_jobs = []
-        for i in range(2, 7):
-            key = f"job{i}"
-            if key in npc._raw_attributes:
-                additional_jobs.append(clean_links(npc._raw_attributes[key]))
-        if additional_jobs:
-            npc.job_additionals = ",".join(additional_jobs)
+        npc: cls = super().from_article(article)
         if npc is None:
             return None
+        npc._parse_jobs()
+        npc._parse_races()
         if "buys" in npc._raw_attributes and article.title != "Minzy":
             cls._parse_buy_offers(npc)
         if "sells" in npc._raw_attributes and article.title != "Minzy":
@@ -268,6 +264,24 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
             npc.destinations.append(NpcDestination(npc_id=npc.article_id, name=destination, price=price, notes=notes))
         return npc
 
+    def _parse_jobs(self):
+        self.jobs = []
+        if "job" in self._raw_attributes:
+            self.jobs.append(self._raw_attributes["job"])
+        for i in range(2, 7):
+            key = f"job{i}"
+            if key in self._raw_attributes:
+                self.jobs.append(clean_links(self._raw_attributes[key]))
+
+    def _parse_races(self):
+        self.races = []
+        if "race" in self._raw_attributes:
+            self.races.append(self._raw_attributes["race"])
+        for i in range(2, 7):
+            key = f"race{i}"
+            if key in self._raw_attributes:
+                self.races.append(clean_links(self._raw_attributes[key]))
+
     @classmethod
     def _parse_buy_offers(cls, npc):
         buy_items = parse_item_offers(npc._raw_attributes["buys"])
@@ -324,8 +338,7 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
                 sorcerer = "sorcerer" in group.lower() or npc.name == "Eliza"
                 if not(knight or paladin or druid or sorcerer):
                     def in_jobs(vocation, _npc):
-                        return vocation in (_npc.job + _npc._raw_attributes.get("job2", "")
-                                            + _npc._raw_attributes.get("job3", "")).lower()
+                        return vocation in ''.join(npc.jobs).lower()
 
                     knight = in_jobs("knight", npc)
                     paladin = in_jobs("paladin", npc)
@@ -354,6 +367,10 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
             spell.insert(c)
         for destination in getattr(self, "destinations", []):
             destination.insert(c)
+        for job in getattr(self, "jobs", []):
+            NpcJob(npc_id=self.article_id, name=job).insert(c)
+        for race in getattr(self, "races", []):
+            NpcRace(npc_id=self.article_id, name=race).insert(c)
 
     @classmethod
     def get_by_field(cls, c, field, value, use_like=False):
@@ -364,10 +381,54 @@ class Npc(abc.Row, abc.Parseable, table=schema.Npc):
         npc.buy_offers = NpcBuyOffer.search(c, "npc_id", npc.article_id, sort_by="value", ascending=False)
         npc.teaches = NpcSpell.search(c, "npc_id", npc.article_id)
         npc.destinations = NpcDestination.search(c, "npc_id", npc.article_id)
+        npc.jobs = [j.name for j in NpcJob.search(c, "npc_id", npc.article_id)]
+        npc.races = [r.name for r in NpcRace.search(c, "npc_id", npc.article_id)]
         return npc
 
 
+class NpcJob(abc.Row, table=schema.NpcJob):
+    """Represents a NPC job.
+
+    This class is only defined internally for parsing purposes. On runtime, this data is attached to the model as a
+    plain string list.
+    """
+
+    __slots__ = (
+        'npc_id',
+        'name',
+    )
+
+    def insert(self, c):
+        columns = {
+            'npc_id': self.npc_id,
+            'name': self.name,
+        }
+        self.table.insert(c, **columns)
+
+
+class NpcRace(abc.Row, table=schema.NpcRace):
+    """Represents an NPC's race.
+
+    This class is only defined internally for parsing purposes. On runtime, this data is attached to the model as a
+    plain string list.
+    """
+
+    __slots__ = (
+        'npc_id',
+        'name',
+    )
+
+    def insert(self, c):
+        columns = {
+            'npc_id': self.npc_id,
+            'name': self.name,
+        }
+        self.table.insert(c, **columns)
+
+
 class NpcOffer:
+    """Represents an NPC buy or sell offer."""
+
     def __init__(self, **kwargs):
         self.npc_id = kwargs.get("npc_id")
         self.npc_title = kwargs.get("npc_title")
@@ -392,8 +453,7 @@ class NpcOffer:
 
 
 class NpcSellOffer(NpcOffer, abc.Row, table=schema.NpcSelling):
-    """
-    Represents an item sellable by an NPC.
+    """Represents an item sellable by an NPC.
 
     Attributes
     ----------
@@ -414,6 +474,7 @@ class NpcSellOffer(NpcOffer, abc.Row, table=schema.NpcSelling):
     value: :class:`str`
         The value of the item in the specified currency.
     """
+
     __slots__ = ("npc_id", "npc_title", "npc_city", "item_id", "item_title", "value", "currency_id", "currency_title")
 
     def __init__(self, **kwargs):
@@ -458,28 +519,28 @@ class NpcSellOffer(NpcOffer, abc.Row, table=schema.NpcSelling):
 
 
 class NpcBuyOffer(NpcOffer, abc.Row, table=schema.NpcBuying):
+    """Represents an item buyable by an NPC.
+
+    Attributes
+    ----------
+    npc_id: :class:`int`
+        The article id of the npc that buys the item.
+    npc_title: :class:`str`
+        The title of the npc that buys the item.
+    npc_city: :class:`str`
+        The city where the NPC is located.
+    item_id: :class:`int`
+        The id of the item bought by the npc.
+    item_title: :class:`str`
+        The title of the item bought by the npc.
+    currency_id: :class:`int`
+        The item id of the currency used to sell the item.
+    currency_title: :class:`str`
+        The title of the currency used to sell the item
+    value: :class:`str`
+        The value of the item in the specified currency.
     """
-        Represents an item buyable by an NPC.
 
-        Attributes
-        ----------
-        npc_id: :class:`int`
-            The article id of the npc that buys the item.
-        npc_title: :class:`str`
-            The title of the npc that buys the item.
-        npc_city: :class:`str`
-            The city where the NPC is located.
-        item_id: :class:`int`
-            The id of the item bought by the npc.
-        item_title: :class:`str`
-            The title of the item bought by the npc.
-        currency_id: :class:`int`
-            The item id of the currency used to sell the item.
-        currency_title: :class:`str`
-            The title of the currency used to sell the item
-        value: :class:`str`
-            The value of the item in the specified currency.
-        """
     __slots__ = ("npc_id", "npc_title", "npc_city", "item_id", "item_title", "value", "currency_id", "currency_title")
 
     def __init__(self, **kwargs):
@@ -523,8 +584,7 @@ class NpcBuyOffer(NpcOffer, abc.Row, table=schema.NpcBuying):
 
 
 class NpcSpell(abc.Row, table=schema.NpcSpell):
-    """
-    Represents a spell that a NPC can teach.
+    """Represents a spell that a NPC can teach.
 
     Attributes
     ----------
@@ -549,6 +609,7 @@ class NpcSpell(abc.Row, table=schema.NpcSpell):
     sorcerer: :class:`bool`
         If the spell is taught to sorcerers.
     """
+
     __slots__ = ("npc_id", "npc_title", "npc_city", "spell_id", "spell_title", "price", "knight", "sorcerer",
                  "paladin", "druid")
 
@@ -595,8 +656,7 @@ class NpcSpell(abc.Row, table=schema.NpcSpell):
 
 
 class NpcDestination(abc.Row, table=schema.NpcDestination):
-    """
-    Represents a NPC's travel destination
+    """Represents a NPC's travel destination.
 
     Attributes
     ----------
@@ -609,6 +669,7 @@ class NpcDestination(abc.Row, table=schema.NpcDestination):
     notes: :class:`str`
         Notes about the destination, such as requirements.
     """
+
     __slots__ = (
         "npc_id",
         "name",
@@ -638,6 +699,7 @@ class RashidPosition(abc.Row, table=schema.RashidPosition):
     location: :class:`str`
         The location where Rashid is that day.
     """
+
     __slots__ = (
         "day",
         "x",
@@ -658,5 +720,5 @@ rashid_positions = [
     RashidPosition(day=3, x=33068, y=32879, z=6, city="Ankrahmun", location="Arito's tavern, above the post office."),
     RashidPosition(day=4, x=33239, y=32480, z=7, city="Darashia", location="Miraia's tavern, south of the guildhalls."),
     RashidPosition(day=5, x=33172, y=31813, z=6, city="Edron", location="Mirabell's tavern, above the depot."),
-    RashidPosition(day=6, x=32326, y=31784, z=6, city="Carlin", location="Carlin depot, one floor above.")
+    RashidPosition(day=6, x=32326, y=31784, z=6, city="Carlin", location="Carlin depot, one floor above."),
 ]
